@@ -1,5 +1,6 @@
 //! 
 //! No-STD abstraction layer enabling numerical functions to be implemented once, and simultaneously support both fixed and floating point types.
+//! The crate focueses on computationally efficient implementations of numerical operations.
 //! 
 //! This is an experimental library.
 //! 
@@ -7,71 +8,16 @@
 #![crate_name = "mixed_num"]
 #![no_std]
 
+#[cfg(feature = "std")]
+extern crate std;
+
 use fixed;
 use num::traits::float::FloatCore;
-use fixed_trigonometry as trig;
 
-mod trigonometry;
+pub mod trigonometry;
 
-pub trait MixedNumConversion<T> {
-    /// Generic type cast from numeric type T.
-    fn mixed_from_num( number:T ) -> Self;
-    /// Generic type cast to  numeric type T.
-    fn mixed_to_num( &self )      -> T;
-}
-
-pub trait MixedTrigonometry
-{
-    /// Take the sin of x. Implementation varies with type.
-    fn mixed_sin(&self) -> Self;
-    /// Take the sin of x. Implementation varies with type.
-    fn mixed_cos(&self) -> Self;
-    /// Take the sin of x. Implementation varies with type.
-    fn mixed_atan(&self) -> Self;
-}
-
-pub trait MixedConsts
-{
-    /// The mixed_pi constant. 3.141...
-    fn mixed_pi() -> Self;
-}
-
-pub trait MixedNum
-    where Self: MixedConsts 
-                + MixedNumConversion<i32> + MixedNumConversion<i64>
-                + MixedNumConversion<f32> + MixedNumConversion<f64>
-                //+ MixedTrigonometry // TODO
-                + core::cmp::PartialOrd
-                + core::marker::Sized
-                + core::ops::Div<Output = Self>
-                + core::ops::Sub<Output = Self>
-                + core::ops::Add<Output = Self>
-                + core::ops::Mul<Output = Self>
-                + core::ops::AddAssign
-                + core::ops::SubAssign
-                + Copy
-{
-    /// Maximum value of the type.
-    fn mixed_max_value() -> Self;
-    /// Minimum value of the type.
-    fn mixed_min_value() -> Self;
-    /// Absolute value.
-    fn mixed_abs( &self ) -> Self;
-    /// Integer valued power.
-    fn mixed_powi( &self, exp: i32 ) -> Self;
-    /// Get the sign of the argument with a unit value.
-    /// Zero is of positive sign.
-    fn mixed_sign( &self ) -> Self;
-    /// Returns a bool if self is positive.
-    fn mixed_is_positive( &self ) -> bool;
-    /// Returns a bool if self is negative.
-    fn mixed_is_negative( &self ) -> bool;
-}
-
-pub trait MixedNumSigned
-    where Self: core::ops::Neg<Output = Self>
-{
-}
+pub mod traits;
+use traits::*;
 
 macro_rules! impl_mixed_num_for_primitive{
     ( $T:ty ) => {
@@ -100,6 +46,30 @@ macro_rules! impl_mixed_num_for_primitive{
             }
         }
 
+        impl MixedNumConversion<u32> for $T
+        {
+            #[inline(always)]
+            fn mixed_from_num( number:u32 ) -> Self {
+                return number as Self;
+            }
+            #[inline(always)]
+            fn mixed_to_num( &self ) -> u32 {
+                return *self as u32;
+            }
+        }
+
+        impl MixedNumConversion<u64> for $T
+        {
+            #[inline(always)]
+            fn mixed_from_num( number:u64 ) -> Self {
+                return number as Self;
+            }
+            #[inline(always)]
+            fn mixed_to_num( &self ) -> u64 {
+                return *self as u64;
+            }
+        }
+
         impl MixedNumConversion<i32> for $T
         {
             #[inline(always)]
@@ -124,27 +94,106 @@ macro_rules! impl_mixed_num_for_primitive{
             }
         }
 
-        /*  // TODO
         impl MixedTrigonometry for $T
         {
-            /// Take the sin of self. Implementation varies with type.
+            /// Take the sin of self.
+            #[cfg(not(feature="std"))]
             #[inline(always)]
             fn mixed_sin(&self) -> Self {
-
+                return trigonometry::sin(*self);
             }
-            /// Take the cos of self. Implementation varies with type.
+            /// Take the cos of self.
+            #[cfg(not(feature="std"))]
             #[inline(always)]
             fn mixed_cos(&self) -> Self {
-
+                return trigonometry::cos(*self);
             }
-            /// Take the sin of self. Implementation varies with type.
+            /// Take the atan of self.
+            #[cfg(not(feature="std"))]
             #[inline(always)]
             fn mixed_atan(&self) -> Self {
+                return trigonometry::atan::atan(*self);
+            }
+            /// Take the atan of self.
+            #[cfg(not(feature="std"))]
+            #[inline(always)]
+            fn mixed_atan2(&self, other:Self) -> Self {
+                return trigonometry::atan::atan2(*self, other);
+            }
 
+            /// Take the sin of self.
+            #[cfg(feature="std")]
+            #[inline(always)]
+            fn mixed_sin(&self) -> Self {
+                return self.sin();
+            }
+            /// Take the cos of self.
+            #[cfg(feature="std")]
+            #[inline(always)]
+            fn mixed_cos(&self) -> Self {
+                return self.cos();
+            }
+            /// Take the atan of self.
+            #[cfg(feature="std")]
+            #[inline(always)]
+            fn mixed_atan(&self) -> Self {
+                return self.atan();
+            }
+            /// Take the atan of self.
+            #[cfg(feature="std")]
+            #[inline(always)]
+            fn mixed_atan2(&self, other:Self) -> Self {
+                return self.atan2(self, other);
             }
         }
-        */
 
+        impl MixedSqrt for $T
+        {
+            /// Take the square root of self.
+            #[cfg(not(feature="std"))]
+            #[inline(always)]
+            fn mixed_sqrt(&self) -> Self {
+                return trigonometry::sqrt::niirf(*self, 2);
+            }
+
+            /// Take the square root of self.
+            #[cfg(feature="std")]
+            #[inline(always)]
+            fn mixed_sqrt(&self) -> Self {
+                return Self.sqrt();
+            }
+
+            /// Take the square root of self.
+            #[inline(always)]
+            fn mixed_niirf(&self) -> Self {
+                return trigonometry::sqrt::niirf(*self, 2);
+            }
+        }
+
+        impl MixedWrapPhase for $T
+        {
+            #[inline(always)]
+            fn mixed_wrap_phase(&self) -> Self {
+                return trigonometry::wrap_phase(*self);
+            }
+        }
+
+        impl MixedOps for $T
+        {
+            #[inline(always)]
+            fn mixed_abs( &self ) -> Self {
+                return self.abs();
+            }
+            #[inline(always)]
+            fn mixed_powi( &self, exp: i32 ) -> Self {
+                return self.powi( exp );
+            }
+        }
+
+        impl MixedNumSigned for $T
+        {   
+        }
+        
         impl MixedNum for $T
         {
             #[inline(always)]
@@ -155,14 +204,6 @@ macro_rules! impl_mixed_num_for_primitive{
             fn mixed_min_value() -> Self {
                 return Self::min_value();
             }
-            #[inline(always)]
-            fn mixed_abs( &self ) -> Self {
-                return self.abs();
-            }
-            #[inline(always)]
-            fn mixed_powi( &self, exp: i32 ) -> Self {
-                return self.powi( exp );
-            }
             fn mixed_sign( &self) -> Self {
                 return trigonometry::sign(*self);
             }
@@ -176,13 +217,35 @@ macro_rules! impl_mixed_num_for_primitive{
             }
         }
 
-        impl MixedConsts for $T
+        impl MixedZero for $T
+        {
+            /// Return the zero value of type Self.
+            #[inline(always)]
+            fn mixed_zero() -> Self {
+                return 0 as $T;
+            }
+        }
+
+        impl MixedOne for $T
+        {
+            /// Return the zero value of type Self.
+            #[inline(always)]
+            fn mixed_one() -> Self {
+                return 1 as $T;
+            }
+        }
+
+        impl MixedPi for $T
         {
             #[inline(always)]
             fn mixed_pi() -> Self {
                 return 3.1415926535897932384626433832795028841971693993751058209749445923078164062 as $T;
             }
         }
+
+        impl MixedConsts for $T
+        {
+        }
     }
 }
 
@@ -216,6 +279,30 @@ macro_rules! impl_mixed_num_for_fixed{
             }
         }
 
+        impl MixedNumConversion<u32> for $T
+        {
+            #[inline(always)]
+            fn mixed_from_num( number:u32 ) -> Self {
+                return Self::from_num(number);
+            }
+            #[inline(always)]
+            fn mixed_to_num( &self ) -> u32 {
+                return self.to_num::<u32>();
+            }
+        }
+
+        impl MixedNumConversion<u64> for $T
+        {
+            #[inline(always)]
+            fn mixed_from_num( number:u64 ) -> Self {
+                return Self::from_num(number);
+            }
+            #[inline(always)]
+            fn mixed_to_num( &self ) -> u64 {
+                return self.to_num::<u64>();
+            }
+        }
+
         impl MixedNumConversion<i32> for $T
         {
             #[inline(always)]
@@ -240,13 +327,49 @@ macro_rules! impl_mixed_num_for_fixed{
             }
         }
 
-        impl MixedConsts for $T
+        impl MixedPi for $T
         {
             #[inline(always)]
             fn mixed_pi() -> Self {
                 return Self::from_num(3.1415926535897932384626433832795028841971693993751058209749445923078164062);
             }
         }
+
+        impl MixedZero for $T
+        {
+            /// Return the zero value of type Self.
+            #[inline(always)]
+            fn mixed_zero() -> Self {
+                return Self::from_num(0) as $T;
+            }
+        }
+
+        impl MixedOne for $T
+        {
+            /// Return the zero value of type Self.
+            #[inline(always)]
+            fn mixed_one() -> Self {
+                return Self::from_num(1) as $T;
+            }
+        }
+
+        impl MixedConsts for $T
+        {
+        }
+
+        impl MixedSqrt for $T
+        {
+            /// Take the square root of self.
+            #[inline(always)]
+            fn mixed_sqrt(&self) -> Self {
+                return trigonometry::sqrt::niirf(*self, 2);
+            }
+            /// Take the square root of self.
+            #[inline(always)]
+            fn mixed_niirf(&self) -> Self {
+                return trigonometry::sqrt::niirf(*self, 2);
+            }
+        }
     }
 }
 
@@ -254,6 +377,18 @@ macro_rules! impl_mixed_num_for_fixed_unsigned{
     ( $T:ty ) => {
         impl_mixed_num_for_fixed!($T);
 
+        impl MixedOps for $T
+        {
+            #[inline(always)]
+            fn mixed_abs( &self ) -> Self {
+                return *self; // Is itself for unsigned.
+            }
+            #[inline(always)]
+            fn mixed_powi( &self, exp: i32 ) -> Self {
+                return trigonometry::powi( *self, exp as usize );
+            }
+        }
+
         impl MixedNum for $T
         {
             #[inline(always)]
@@ -265,14 +400,6 @@ macro_rules! impl_mixed_num_for_fixed_unsigned{
                 return Self::MIN;
             }
             #[inline(always)]
-            fn mixed_abs( &self ) -> Self {
-                return *self; // Is itself for unsigned.
-            }
-            #[inline(always)]
-            fn mixed_powi( &self, exp: i32 ) -> Self {
-                return trig::powi( *self, exp as usize );
-            }
-            #[inline(always)]
             fn mixed_sign( &self) -> Self {
                 return trigonometry::sign(*self);
             }
@@ -292,6 +419,22 @@ macro_rules! impl_mixed_num_for_fixed_signed{
     ( $T:ty ) => {
         impl_mixed_num_for_fixed!($T);
 
+        impl MixedNumSigned  for $T
+        {
+        }
+
+        impl MixedOps for $T
+        {
+            #[inline(always)]
+            fn mixed_abs( &self ) -> Self {
+                return self.abs();
+            }
+            #[inline(always)]
+            fn mixed_powi( &self, exp: i32 ) -> Self {
+                return trigonometry::powi( *self, exp as usize );
+            }
+        }
+
         impl MixedNum for $T
         {
             #[inline(always)]
@@ -303,14 +446,6 @@ macro_rules! impl_mixed_num_for_fixed_signed{
                 return Self::MIN;
             }
             #[inline(always)]
-            fn mixed_abs( &self ) -> Self {
-                return self.abs();
-            }
-            #[inline(always)]
-            fn mixed_powi( &self, exp: i32 ) -> Self {
-                return trig::powi( *self, exp as usize );
-            }
-            #[inline(always)]
             fn mixed_sign( &self) -> Self {
                 return trigonometry::sign(*self);
             }
@@ -323,19 +458,50 @@ macro_rules! impl_mixed_num_for_fixed_signed{
                 return self.is_negative();
             }
         }
+
+        impl MixedWrapPhase for $T
+        {
+            #[inline(always)]
+            fn mixed_wrap_phase(&self) -> Self {
+                return trigonometry::wrap_phase(*self);
+            }
+        }
+
+        impl MixedTrigonometry for $T
+        {
+            /// Take the sin of self. Implementation varies with type.
+            #[inline(always)]
+            fn mixed_sin(&self) -> Self {
+                return cordic::sin(*self);
+            }
+            /// Take the cos of self. Implementation varies with type.
+            #[inline(always)]
+            fn mixed_cos(&self) -> Self {
+                return cordic::cos(*self);
+            }
+            /// Take the atan of self. Implementation varies with type.
+            #[inline(always)]
+            fn mixed_atan(&self) -> Self {
+                return trigonometry::atan::atan(*self);
+            }
+            /// Take the atan of self. Implementation varies with type.
+            #[inline(always)]
+            fn mixed_atan2(&self, other:Self) -> Self {
+                return trigonometry::atan::atan2(*self, other);
+            }
+        }
     }
 }
 
-
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU8<fixed::types::extra::U0>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU8<fixed::types::extra::U1>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU8<fixed::types::extra::U2>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU8<fixed::types::extra::U3>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU8<fixed::types::extra::U4>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU8<fixed::types::extra::U5>);
-impl_mixed_num_for_fixed_unsigned!(fixed::FixedU8<fixed::types::extra::U6>);
-impl_mixed_num_for_fixed_unsigned!(fixed::FixedU8<fixed::types::extra::U7>);
-impl_mixed_num_for_fixed_unsigned!(fixed::FixedU8<fixed::types::extra::U8>);
+//impl_mixed_num_for_fixed_unsigned!(fixed::FixedU8<fixed::types::extra::U6>);
+//impl_mixed_num_for_fixed_unsigned!(fixed::FixedU8<fixed::types::extra::U7>);
+//impl_mixed_num_for_fixed_unsigned!(fixed::FixedU8<fixed::types::extra::U8>);
 
 impl_mixed_num_for_fixed_signed!(fixed::FixedI8<fixed::types::extra::U0>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI8<fixed::types::extra::U1>);
@@ -343,9 +509,9 @@ impl_mixed_num_for_fixed_signed!(fixed::FixedI8<fixed::types::extra::U2>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI8<fixed::types::extra::U3>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI8<fixed::types::extra::U4>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI8<fixed::types::extra::U5>);
-impl_mixed_num_for_fixed_signed!(fixed::FixedI8<fixed::types::extra::U6>);
-impl_mixed_num_for_fixed_signed!(fixed::FixedI8<fixed::types::extra::U7>);
-impl_mixed_num_for_fixed_signed!(fixed::FixedI8<fixed::types::extra::U8>);
+//impl_mixed_num_for_fixed_signed!(fixed::FixedI8<fixed::types::extra::U6>);
+//impl_mixed_num_for_fixed_signed!(fixed::FixedI8<fixed::types::extra::U7>);
+//impl_mixed_num_for_fixed_signed!(fixed::FixedI8<fixed::types::extra::U8>);
 
 
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU16<fixed::types::extra::U0>);
@@ -362,9 +528,9 @@ impl_mixed_num_for_fixed_unsigned!(fixed::FixedU16<fixed::types::extra::U10>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU16<fixed::types::extra::U11>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU16<fixed::types::extra::U12>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU16<fixed::types::extra::U13>);
-impl_mixed_num_for_fixed_unsigned!(fixed::FixedU16<fixed::types::extra::U14>);
-impl_mixed_num_for_fixed_unsigned!(fixed::FixedU16<fixed::types::extra::U15>);
-impl_mixed_num_for_fixed_unsigned!(fixed::FixedU16<fixed::types::extra::U16>);
+//impl_mixed_num_for_fixed_unsigned!(fixed::FixedU16<fixed::types::extra::U14>);
+//impl_mixed_num_for_fixed_unsigned!(fixed::FixedU16<fixed::types::extra::U15>);
+//impl_mixed_num_for_fixed_unsigned!(fixed::FixedU16<fixed::types::extra::U16>);
 
 impl_mixed_num_for_fixed_signed!(fixed::FixedI16<fixed::types::extra::U0>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI16<fixed::types::extra::U1>);
@@ -380,9 +546,9 @@ impl_mixed_num_for_fixed_signed!(fixed::FixedI16<fixed::types::extra::U10>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI16<fixed::types::extra::U11>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI16<fixed::types::extra::U12>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI16<fixed::types::extra::U13>);
-impl_mixed_num_for_fixed_signed!(fixed::FixedI16<fixed::types::extra::U14>);
-impl_mixed_num_for_fixed_signed!(fixed::FixedI16<fixed::types::extra::U15>);
-impl_mixed_num_for_fixed_signed!(fixed::FixedI16<fixed::types::extra::U16>);
+//impl_mixed_num_for_fixed_signed!(fixed::FixedI16<fixed::types::extra::U14>);
+//impl_mixed_num_for_fixed_signed!(fixed::FixedI16<fixed::types::extra::U15>);
+//impl_mixed_num_for_fixed_signed!(fixed::FixedI16<fixed::types::extra::U16>);
 
 
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU32<fixed::types::extra::U0>);
@@ -415,9 +581,9 @@ impl_mixed_num_for_fixed_unsigned!(fixed::FixedU32<fixed::types::extra::U26>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU32<fixed::types::extra::U27>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU32<fixed::types::extra::U28>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU32<fixed::types::extra::U29>);
-impl_mixed_num_for_fixed_unsigned!(fixed::FixedU32<fixed::types::extra::U30>);
-impl_mixed_num_for_fixed_unsigned!(fixed::FixedU32<fixed::types::extra::U31>);
-impl_mixed_num_for_fixed_unsigned!(fixed::FixedU32<fixed::types::extra::U32>);
+//impl_mixed_num_for_fixed_unsigned!(fixed::FixedU32<fixed::types::extra::U30>);
+//impl_mixed_num_for_fixed_unsigned!(fixed::FixedU32<fixed::types::extra::U31>);
+//impl_mixed_num_for_fixed_unsigned!(fixed::FixedU32<fixed::types::extra::U32>);
 
 impl_mixed_num_for_fixed_signed!(fixed::FixedI32<fixed::types::extra::U0>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI32<fixed::types::extra::U1>);
@@ -449,9 +615,9 @@ impl_mixed_num_for_fixed_signed!(fixed::FixedI32<fixed::types::extra::U26>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI32<fixed::types::extra::U27>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI32<fixed::types::extra::U28>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI32<fixed::types::extra::U29>);
-impl_mixed_num_for_fixed_signed!(fixed::FixedI32<fixed::types::extra::U30>);
-impl_mixed_num_for_fixed_signed!(fixed::FixedI32<fixed::types::extra::U31>);
-impl_mixed_num_for_fixed_signed!(fixed::FixedI32<fixed::types::extra::U32>);
+//impl_mixed_num_for_fixed_signed!(fixed::FixedI32<fixed::types::extra::U30>);
+//impl_mixed_num_for_fixed_signed!(fixed::FixedI32<fixed::types::extra::U31>);
+//impl_mixed_num_for_fixed_signed!(fixed::FixedI32<fixed::types::extra::U32>);
 
 
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU64<fixed::types::extra::U0>);
@@ -516,9 +682,9 @@ impl_mixed_num_for_fixed_unsigned!(fixed::FixedU64<fixed::types::extra::U58>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU64<fixed::types::extra::U59>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU64<fixed::types::extra::U60>);
 impl_mixed_num_for_fixed_unsigned!(fixed::FixedU64<fixed::types::extra::U61>);
-impl_mixed_num_for_fixed_unsigned!(fixed::FixedU64<fixed::types::extra::U62>);
-impl_mixed_num_for_fixed_unsigned!(fixed::FixedU64<fixed::types::extra::U63>);
-impl_mixed_num_for_fixed_unsigned!(fixed::FixedU64<fixed::types::extra::U64>);
+//impl_mixed_num_for_fixed_unsigned!(fixed::FixedU64<fixed::types::extra::U62>);
+//impl_mixed_num_for_fixed_unsigned!(fixed::FixedU64<fixed::types::extra::U63>);
+//impl_mixed_num_for_fixed_unsigned!(fixed::FixedU64<fixed::types::extra::U64>);
 
 impl_mixed_num_for_fixed_signed!(fixed::FixedI64<fixed::types::extra::U0>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI64<fixed::types::extra::U1>);
@@ -582,6 +748,6 @@ impl_mixed_num_for_fixed_signed!(fixed::FixedI64<fixed::types::extra::U58>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI64<fixed::types::extra::U59>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI64<fixed::types::extra::U60>);
 impl_mixed_num_for_fixed_signed!(fixed::FixedI64<fixed::types::extra::U61>);
-impl_mixed_num_for_fixed_signed!(fixed::FixedI64<fixed::types::extra::U62>);
-impl_mixed_num_for_fixed_signed!(fixed::FixedI64<fixed::types::extra::U63>);
-impl_mixed_num_for_fixed_signed!(fixed::FixedI64<fixed::types::extra::U64>);
\ No newline at end of file
+//impl_mixed_num_for_fixed_signed!(fixed::FixedI64<fixed::types::extra::U62>);
+//impl_mixed_num_for_fixed_signed!(fixed::FixedI64<fixed::types::extra::U63>);
+//impl_mixed_num_for_fixed_signed!(fixed::FixedI64<fixed::types::extra::U64>);
\ No newline at end of file
